#include "Writer.hpp"
#include <stdio.h>
#include <iostream>
#include<fstream>
#include<iomanip>
#include<cmath>

#define FIFO_BUFFER_MAX_SIZE 65535
#define MAX_WRITE_ERRORS 1000
#define WITNESS_FILE_NAME "witness"

using namespace std;

Writer::Writer(const QByteArray& inputData, char* pipe) : m_inputData(inputData), m_pipe(pipe), QThread() {}

void Writer::run() {
    int nbBytesWritten, nbBytesWrittenForWitness, fd, ret;
    int nbWrittenBytesTotal = 0, nbCycles = 0, nbWriteErrors = 0;
    int byteStreamSize = m_inputData.size();
    int nbCyclesTheoritical = byteStreamSize / FIFO_BUFFER_MAX_SIZE + ((byteStreamSize % FIFO_BUFFER_MAX_SIZE) > 0 ? 1 : 0);
    int nbLeadingSpacesCycles = log10(nbCyclesTheoritical) + 1; // used for formatting logs
    int nbLeadingSpacesBytes = log10(byteStreamSize) + 1; // used for formatting logs
    cout << "[Writer] processing a " << byteStreamSize << " byte stream (theorically in " << nbCyclesTheoritical << " write cycle" << (nbCyclesTheoritical > 1 ? "s)" : ")") << endl;

    // open "witness" file (used to check that the byte stream is strictly identical to the content of the file being processed
    FILE *file = fopen(WITNESS_FILE_NAME, "w");
    if (file == NULL) {
	cerr << "[Writer] error opening file for writing - resuming execution" << endl;
    }

    // open pipe for writing
    if ((fd = open(m_pipe, O_WRONLY)) < 0) {
       cout << "[Writer] error opening pipe named " << m_pipe << " (O_WRONLY) - " << strerror(errno) << endl;
       return;
    }

    // write loop
    while (true) {
	// Write next segment of FIFO_BUFFER_MAX_SIZE bytes
	if (nbWrittenBytesTotal < byteStreamSize) {
		nbCycles++;
		cout << "[Writer] \\_ cycle " << std::setw(nbLeadingSpacesCycles) << std::setfill(' ') << nbCycles << " / " << std::setw(nbLeadingSpacesCycles) << std::setfill(' ') << nbCyclesTheoritical << ends;
		cout << ": writing segment [" << std::setw(nbLeadingSpacesBytes) << std::setfill(' ') << nbWrittenBytesTotal << ", " << std::setw(nbLeadingSpacesBytes) << std::setfill(' ') << nbWrittenBytesTotal + FIFO_BUFFER_MAX_SIZE - 1 << "]... " << ends;
	}
        QByteArray segment = m_inputData.mid(nbWrittenBytesTotal, FIFO_BUFFER_MAX_SIZE);
	nbBytesWritten = write(fd, segment.data(), segment.length());
	if (nbWrittenBytesTotal < byteStreamSize) cout << "bytes written into pipe: " << nbBytesWritten << endl;
	// Write byte stream into "witness" file
	int nbBytesWrittenForWitness = fwrite(segment.data(), sizeof(char), segment.length(), file);
	// check number of bytes actually written into pipe
        if (nbBytesWritten < 0) {
	    if (++nbWriteErrors > MAX_WRITE_ERRORS) {
		cerr << "[Writer] \\_ cycle " << std::setw(nbLeadingSpacesCycles) << std::setfill(' ') << nbCycles << " / " << std::setw(5) << std::setfill(' ') << nbCyclesTheoritical << ends;
		cerr << ": too many errors occurred writing into pipe (" << nbWrittenBytesTotal << " byte" << (nbWrittenBytesTotal > 1 ? "s" : "") << " written in " << nbCycles << " cycle" << (nbCycles > 1 ? "s" : "") << ends;
		cerr << " so far) - " << strerror(errno) << " - abort writing" << endl;
		break;
	    }
	    cerr << "[Writer] \\_ cycle " << std::setw(nbLeadingSpacesCycles) << std::setfill(' ') << nbCycles << " / " << std::setw(5) << std::setfill(' ') << nbCyclesTheoritical << ends;
	    cerr << ": error writing into pipe (" << nbWrittenBytesTotal << " byte" << (nbWrittenBytesTotal > 1 ? "s" : "") << " written in " << nbCycles << " cycle" << (nbCycles > 1 ? "s" : "") << ends;
	    cerr << " so far) - " << strerror(errno) << endl;
        } else {
	  if (nbBytesWritten == 0) {
	    cout << "[Writer] finished writing into " << m_pipe << " after having sent " << nbWrittenBytesTotal << " byte" << (nbWrittenBytesTotal > 1 ? "s" : "") << " in " << nbCycles << " cycle" << (nbCycles > 1 ? "s" : "") << endl;
	    break;
	  }
	  nbWrittenBytesTotal += nbBytesWritten;
	}
    }
    if ((ret = close(fd)) < 0) cout << "[Writer] error closing pipe (O_WRONLY): " << strerror(errno) << endl;
    if ((ret = fclose(file)) < 0) cerr << "[Writer] error closing file: " << strerror(errno) << endl;
}
