#include <stdio.h>
#include <iostream>
#include <vector>

#include <cstdlib>
#include <sys/stat.h>

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "ExifTool.h"
#include "Writer.hpp"

#include <QFile>
#include <QByteArray>

using namespace std;

#define PIPE_NAME "myPipe"

int main(int argc, char **argv)
{
    int ret;
    if (argc < 2) {
        cout << "--- process a file with exiftool through a named pipe ---" << endl;
        cout << "I need the name of the file you want to process." << endl;
        return 1;
    }
    cout << "[main] start" << endl;

    // create our ExifTool object
    ExifTool *et = new ExifTool();

    // get file content into a byte array
    QFile file(argv[1]);
    if (!file.open(QIODevice::ReadOnly)) {
	cerr << "[main] error opening file " << argv[1] << endl;
	return 1;
    } else cout << "[main] processing file " << argv[1] << endl;
    QByteArray inputData = file.readAll();
    cout << "[main] inputData.size() = " << inputData.size() << " byte" << (inputData.size() > 1 ? "s" : "") << endl;

    // create named pipe
    char* pipe = const_cast<char *>(PIPE_NAME);
    struct stat buffer;
    // If pipe already exists, remove it
    if (stat(pipe, &buffer) == 0) {
	ret = unlink(pipe); // remove pipe
	if (ret != 0) {
		cerr << "[main] pipe named " << pipe << " already exists, and cannot be removed!" << endl;
		cerr << "[main] aborted" << endl;
		return 1;
	}
    }
    ret = mkfifo(pipe, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
    if (ret != 0) {
	cerr << "[main] error creating pipe named " << pipe << " (return code: " << ret << " - " << strerror(errno) << ")" << endl;
	cerr << "[main] aborted" << endl;
	return 1;
    }

    // open pipe for reading
    int fd;
    if ((fd = open(pipe, O_RDONLY|O_NONBLOCK)) < 0) {
	cerr << "[main] error opening pipe named " << pipe << " (O_RDONLY and O_NONBLOCK) - " << strerror(errno) << endl;
	cerr << "[main] aborted" << endl;
	return 1;
    }

    // start writing
    Writer* w = new Writer(inputData, pipe);
    cout << "[main] start writing thread" << endl;
    w->start();

    // read metadata from the image
    cout << "[main] call to ImageInfo (reads from named pipe)" << endl;
    TagInfo *info = et->ImageInfo(pipe,NULL,5);
    if (info) {
	cout << "[main] -" << endl;
	cout << "[main] exiftool has finished and returned the following meta information :" << endl;
        // print returned information
        for (TagInfo *i=info; i; i=i->next) {
            cout << "[main] \\_ " << i->name << " = " << i->value << endl;
        }
	cout << "[main] -" << endl;
        // we are responsible for deleting the information when done
        delete info;
    }
    if (et->LastComplete() <= 0) cerr << "[main] ## error executing exiftool! ##" << endl;
    // print exiftool stderr messages
    char *err = et->GetError();
    if (err) cerr << err;
    else if (et->LastComplete() <= 0) cerr << "[main] ## no error message! ##" << endl;
    ret = unlink(pipe); // suppression du pipe
    if (ret != 0) {
      cerr << "[main] error removing pipe named : " << pipe << " (return code: " << ret << " - " << strerror(errno) << ")" << endl;
      return 1;
    }
    cout << "[main] deleting Exiftool object..." << endl;
    delete et; // delete our ExifTool object
    cout << "[main] Exiftool object deleted" << endl;
    cout << "[main] -- end of execution --" << endl;
    return 0;
}
// end
